@@ -1,16 +1,10 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | Copyright (c) 1997-2013 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000, 2001 The PHP Group             |
-   +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
-   | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
-   | If you did not receive a copy of the PHP license and are unable to   |
-   | obtain it through the world-wide-web, please send a note to          |
-   | license@php.net so we can mail you a copy immediately.               |
+   | This source file is subject to the 2-Clause BSD license which is     |
+   | available through the LICENSE file, or online at                     |
+   | http://opensource.org/licenses/bsd-license.php                       |
    +----------------------------------------------------------------------+
    | Authors:  Derick Rethans <derick@derickrethans.nl>                   |
    +----------------------------------------------------------------------+
@@ -23,6 +17,9 @@
 #include "set.h"
 #include "php_vld.h"
 
+#define VLD_JMP_NOT_SET -1
+#define VLD_JMP_EXIT    -2
+
 typedef struct _vld_branch {
 	unsigned int start_lineno;
 	unsigned int end_lineno;
@@ -38,6 +35,7 @@ typedef struct _vld_path {
 
 typedef struct _vld_branch_info {
 	unsigned int  size;
+	vld_set      *entry_points;
 	vld_set      *starts;
 	vld_set      *ends;
 	vld_branch   *branches;
@@ -50,7 +48,7 @@ typedef struct _vld_branch_info {
 vld_branch_info *vld_branch_info_create(unsigned int size);
 
 void vld_branch_info_update(vld_branch_info *branch_info, unsigned int pos, unsigned int lineno, unsigned int outidx, unsigned int jump_pos);
-void vld_branch_post_process(vld_branch_info *branch_info);
+void vld_branch_post_process(zend_op_array *opa, vld_branch_info *branch_info);
 void vld_branch_find_paths(vld_branch_info *branch_info);
 
 void vld_branch_info_dump(zend_op_array *opa, vld_branch_info *branch_info TSRMLS_DC);
