@@ -1,16 +1,10 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | Copyright (c) 1997-2013 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000, 2001 The PHP Group             |
-   +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
-   | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
-   | If you did not receive a copy of the PHP license and are unable to   |
-   | obtain it through the world-wide-web, please send a note to          |
-   | license@php.net so we can mail you a copy immediately.               |
+   | This source file is subject to the 2-Clause BSD license which is     |
+   | available through the LICENSE file, or online at                     |
+   | http://opensource.org/licenses/bsd-license.php                       |
    +----------------------------------------------------------------------+
    | Authors:  Derick Rethans <derick@derickrethans.nl>                   |
    +----------------------------------------------------------------------+
