@@ -1,16 +1,10 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | Copyright (c) 1997-2013 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000, 2001 The PHP Group             |
-   +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
-   | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
-   | If you did not receive a copy of the PHP license and are unable to   |
-   | obtain it through the world-wide-web, please send a note to          |
-   | license@php.net so we can mail you a copy immediately.               |
+   | This source file is subject to the 2-Clause BSD license which is     |
+   | available through the LICENSE file, or online at                     |
+   | http://opensource.org/licenses/bsd-license.php                       |
    +----------------------------------------------------------------------+
    | Authors:  Derick Rethans <derick@derickrethans.nl>                   |
    |           Andrei Zmievski <andrei@gravitonic.com>                    |
@@ -236,120 +230,127 @@ static const op_usage opcodes[] = {
 	/*  159 */	{ "DISCARD_EXCEPTION", NONE_USED },
 	/*  160 */	{ "YIELD", ALL_USED },
 	/*  161 */	{ "GENERATOR_RETURN", NONE_USED },
-	/*  162 */	{ "FAST_CALL", OP1_USED },
-	/*  163 */	{ "FAST_RET", NONE_USED },
+	/*  162 */	{ "FAST_CALL", SPECIAL },
+	/*  163 */	{ "FAST_RET", SPECIAL },
+	/*  164 */	{ "ZEND_RECV_VARIADIC", ALL_USED },
+	/*  165 */	{ "POW", ALL_USED },
+	/*  166 */	{ "ASSIGN_POW", ALL_USED | EXT_VAL },
 #endif
 };
 
-#if PHP_VERSION_ID >= 50399
-# define VLD_ZNODE znode_op
-# define VLD_ZNODE_ELEM(node,var) node.var
-# define VLD_TYPE(t) t##_type
-# define VLD_EXTENDED_VALUE(o) extended_value
-#else
-# define VLD_ZNODE znode
-# define VLD_ZNODE_ELEM(node,var) node.u.var
-# define VLD_TYPE(t) t.op_type
-# define VLD_EXTENDED_VALUE(o) o.u.EA.type
-#endif
+zend_brk_cont_element* vld_find_brk_cont(int nest_levels, int array_offset, zend_op_array *op_array);
 
-#if PHP_VERSION_ID >= 50500
-# define VAR_NUM(v) ((zend_uint)(EX_TMP_VAR_NUM(0, 0) - EX_TMP_VAR(0, v)))
-#else
-# define VAR_NUM(v) ((v)/(sizeof(temp_variable)))
-#endif
-
-zend_brk_cont_element* vld_find_brk_cont(zend_uint nest_levels, int array_offset, zend_op_array *op_array);
-
-static inline int vld_dump_zval_null(zvalue_value value)
+static inline int vld_dump_zval_null(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "null");
 }
 
-static inline int vld_dump_zval_long(zvalue_value value)
+static inline int vld_dump_zval_long(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "%ld", value.lval);
 }
 
-static inline int vld_dump_zval_double(zvalue_value value)
+static inline int vld_dump_zval_double(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "%g", value.dval);
 }
 
-static inline int vld_dump_zval_string(zvalue_value value)
+static inline int vld_dump_zval_string(ZVAL_VALUE_TYPE value)
 {
-	char *new_str;
+	ZVAL_VALUE_STRING_TYPE *new_str;
 	int new_len, len;
 
-	new_str = php_url_encode(value.str.val, value.str.len, &new_len);
-	len = vld_printf (stderr, "'%s'", new_str);
+	new_str = php_url_encode(ZVAL_STRING_VALUE(value), ZVAL_STRING_LEN(value) PHP_URLENCODE_NEW_LEN(new_len));
+	len = vld_printf (stderr, "'%s'", ZSTRING_VALUE(new_str));
 	efree(new_str);
 	return len;
 }
 
-#if PHP_VERSION_ID >= 60000
-static inline int vld_dump_zval_unicode(zvalue_value value)
-{
-	int len;
-	
-	len = vld_printf(stderr, "%R", IS_UNICODE, value.ustr.val);
-	return len;
-}
-#endif
-
-static inline int vld_dump_zval_array(zvalue_value value)
+static inline int vld_dump_zval_array(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "<array>");
 }
 
-static inline int vld_dump_zval_object(zvalue_value value)
+static inline int vld_dump_zval_object(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "<object>");
 }
 
-static inline int vld_dump_zval_bool(zvalue_value value)
+static inline int vld_dump_zval_bool(ZVAL_VALUE_TYPE value)
 {
-	return vld_printf (stderr, value.lval ? "true" : "false");
+	return vld_printf (stderr, "<bool>");
 }
 
-static inline int vld_dump_zval_resource(zvalue_value value)
+static inline int vld_dump_zval_true(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<true>");
+}
+
+static inline int vld_dump_zval_false(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<false>");
+}
+
+static inline int vld_dump_zval_resource(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "<resource>");
 }
 
-static inline int vld_dump_zval_constant(zvalue_value value)
+static inline int vld_dump_zval_constant(ZVAL_VALUE_TYPE value)
 {
-	return vld_printf (stderr, "<const:'%s'>", value.str.val);
+	return vld_printf (stderr, "<const:'%s'>", ZVAL_STRING_VALUE(value));
 }
 
-static inline int vld_dump_zval_constant_array(zvalue_value value)
+#if PHP_VERSION_ID >= 50600
+static inline int vld_dump_zval_constant_ast(ZVAL_VALUE_TYPE value)
+{
+	return vld_printf (stderr, "<const ast>");
+}
+#else
+static inline int vld_dump_zval_constant_array(ZVAL_VALUE_TYPE value)
 {
 	return vld_printf (stderr, "<const array>");
 }
+#endif
 
 
 int vld_dump_zval (zval val)
 {
+#if PHP_VERSION_ID >= 70000
+	switch (val.u1.v.type) {
+#else
 	switch (val.type) {
+#endif
 		case IS_NULL:           return vld_dump_zval_null (val.value);
 		case IS_LONG:           return vld_dump_zval_long (val.value);
 		case IS_DOUBLE:         return vld_dump_zval_double (val.value);
 		case IS_STRING:         return vld_dump_zval_string (val.value);
 		case IS_ARRAY:          return vld_dump_zval_array (val.value);
 		case IS_OBJECT:         return vld_dump_zval_object (val.value);
-		case IS_BOOL:           return vld_dump_zval_bool (val.value);
 		case IS_RESOURCE:       return vld_dump_zval_resource (val.value);
 		case IS_CONSTANT:       return vld_dump_zval_constant (val.value);
+#if PHP_VERSION_ID >= 50600
+		case IS_CONSTANT_AST:   return vld_dump_zval_constant_ast (val.value);
+#else
 		case IS_CONSTANT_ARRAY: return vld_dump_zval_constant_array (val.value);
-#if PHP_VERSION_ID >= 60000
-		case IS_UNICODE:        return vld_dump_zval_unicode (val.value);
+#endif
+#if PHP_VERSION_ID >= 70000
+		case IS_UNDEF:          return vld_dump_zval_undef (val.value);
+		case IS_FALSE:          return vld_dump_zval_false (val.value);
+		case IS_TRUE:           return vld_dump_zval_true (val.value);
+		case IS_REFERENCE:      return vld_dump_zval_reference (val.value);
+		case IS_CALLABLE:       return vld_dump_zval_callable (val.value);
+		case IS_INDIRECT:       return vld_dump_zval_indirect (val.value);
+		case IS_PTR:            return vld_dump_zval_ptr (val.value);
+#else
+		case IS_BOOL:           return vld_dump_zval_bool (val.value);
 #endif
 	}
 	return vld_printf(stderr, "<unknown>");
 }
 
 
-int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, zend_uint base_address TSRMLS_DC)
+int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, unsigned int base_address TSRMLS_DC)
 {
 	int len = 0;
 
@@ -364,11 +365,19 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, zend
 			VLD_PRINT(3, " IS_UNUSED ");
 			break;
 		case IS_CONST: /* 1 */
+#if PHP_VERSION_ID >= 70000
+			VLD_PRINT1(3, " IS_CONST (%d) ", VLD_ZNODE_ELEM(node, var) / sizeof(zval));
+#else
 			VLD_PRINT1(3, " IS_CONST (%d) ", VLD_ZNODE_ELEM(node, var) / sizeof(temp_variable));
-#if PHP_VERSION_ID >= 50399
+#endif
+#if PHP_VERSION_ID >= 70000
 			vld_dump_zval(*node.zv);
 #else
+# if PHP_VERSION_ID >= 50399
+			vld_dump_zval(*node.zv);
+# else
 			vld_dump_zval(node.u.constant);
+# endif
 #endif
 			break;
 #ifdef ZEND_ENGINE_2
@@ -383,14 +392,26 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, zend
 #if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 1)
 		case IS_CV:  /* 16 */
 			VLD_PRINT(3, " IS_CV ");
+#if PHP_VERSION_ID >= 70000
+			len += vld_printf (stderr, "!%d", (VLD_ZNODE_ELEM(node, var)-sizeof(zend_execute_data)) / sizeof(zval));
+#else
 			len += vld_printf (stderr, "!%d", VLD_ZNODE_ELEM(node, var));
+#endif
 			break;
 #endif
 		case VLD_IS_OPNUM:
+#if PHP_VERSION_ID >= 70000
+			len += vld_printf (stderr, "->%d", ((long) VLD_ZNODE_ELEM(node, jmp_addr) - base_address) / sizeof(zend_op));
+#else
 			len += vld_printf (stderr, "->%d", VLD_ZNODE_ELEM(node, opline_num));
+#endif
 			break;
 		case VLD_IS_OPLINE:
+#if PHP_VERSION_ID >= 70000
+			len += vld_printf (stderr, "->%d", ((long) VLD_ZNODE_ELEM(node, jmp_addr) - base_address) / sizeof(zend_op));
+#else
 			len += vld_printf (stderr, "->%d", (VLD_ZNODE_ELEM(node, opline_num) - base_address) / sizeof(zend_op));
+#endif
 			break;
 		case VLD_IS_CLASS:
 			len += vld_printf (stderr, ":%d", VAR_NUM(VLD_ZNODE_ELEM(node, var)));
@@ -416,9 +437,9 @@ int vld_dump_znode (int *print_sep, unsigned int node_type, VLD_ZNODE node, zend
 	return len;
 }
 
-static zend_uint vld_get_special_flags(const zend_op *op, zend_uint base_address)
+static unsigned int vld_get_special_flags(const zend_op *op, unsigned int base_address)
 {
-	zend_uint flags = 0;
+	unsigned int flags = 0;
 
 	switch (op->opcode) {
 		case ZEND_FE_RESET:
@@ -437,7 +458,9 @@ static zend_uint vld_get_special_flags(const zend_op *op, zend_uint base_address
 			}
 			break;
 
+#if PHP_VERSION_ID < 70000
 		case ZEND_DO_FCALL_BY_NAME:
+#endif
 		case ZEND_DO_FCALL:
 			flags = OP1_USED | RES_USED | EXT_VAL;
 			/*flags = ALL_USED | EXT_VAL;
@@ -483,14 +506,26 @@ static zend_uint vld_get_special_flags(const zend_op *op, zend_uint base_address
 		case ZEND_CONT:
 			flags = OP2_USED|OP2_BRK_CONT;
 			break;
-
+#if PHP_VERSION_ID >= 50500
+		case ZEND_FAST_CALL:
+			flags = OP1_USED|OP1_OPLINE;
+			if (op->extended_value) {
+				flags |= OP2_USED|OP2_OPNUM|EXT_VAL;
+			}
+			break;
+		case ZEND_FAST_RET:
+			if (op->extended_value) {
+				flags = OP2_USED|OP2_OPNUM|EXT_VAL;
+			}
+			break;
+#endif
 	}
 	return flags;
 }
 
 #define NUM_KNOWN_OPCODES (sizeof(opcodes)/sizeof(opcodes[0]))
 
-void vld_dump_op(int nr, zend_op * op_ptr, zend_uint base_address, int notdead, int start, int end, zend_op_array *opa TSRMLS_DC)
+void vld_dump_op(int nr, zend_op * op_ptr, unsigned int base_address, int notdead, int entry, int start, int end, zend_op_array *opa TSRMLS_DC)
 {
 	static uint last_lineno = (uint) -1;
 	int print_sep = 0, len;
@@ -514,13 +549,12 @@ void vld_dump_op(int nr, zend_op * op_ptr, zend_uint base_address, int notdead,
 
 	if (flags == SPECIAL) {
 		flags = vld_get_special_flags(&op, base_address);
-	} else {
-		if (flags & OP1_OPLINE) {
-			op1_type = VLD_IS_OPLINE;
-		}
-		if (flags & OP2_OPLINE) {
-			op2_type = VLD_IS_OPLINE;
-		}
+	} 
+	if (flags & OP1_OPLINE) {
+		op1_type = VLD_IS_OPLINE;
+	}
+	if (flags & OP2_OPLINE) {
+		op2_type = VLD_IS_OPLINE;
 	}
 	if (flags & OP1_OPNUM) {
 		op1_type = VLD_IS_OPNUM;
@@ -582,20 +616,20 @@ void vld_dump_op(int nr, zend_op * op_ptr, zend_uint base_address, int notdead,
 
 	if (op.opcode >= NUM_KNOWN_OPCODES) {
 		if (VLD_G(format)) {
-			vld_printf(stderr, "%5d %s %c %c %c %s <%03d>%-23s %s %-14s ", nr, VLD_G(col_sep), notdead ? ' ' : '*', start ? '>' : ' ', end ? '>' : ' ', VLD_G(col_sep), op.opcode, VLD_G(col_sep), fetch_type);
+			vld_printf(stderr, "%5d %s %c %c %c %c %s <%03d>%-23s %s %-14s ", nr, VLD_G(col_sep), notdead ? ' ' : '*', entry ? 'E' : ' ', start ? '>' : ' ', end ? '>' : ' ', VLD_G(col_sep), op.opcode, VLD_G(col_sep), fetch_type);
 		} else {
-			vld_printf(stderr, "%5d%c %c %c <%03d>%-23s %-14s ", nr, notdead ? ' ' : '*', start ? '>' : ' ', end ? '>' : ' ', op.opcode, "", fetch_type);
+			vld_printf(stderr, "%5d%c %c %c %c <%03d>%-23s %-14s ", nr, notdead ? ' ' : '*', entry ? 'E' : ' ', start ? '>' : ' ', end ? '>' : ' ', op.opcode, "", fetch_type);
 		}
 	} else {
 		if (VLD_G(format)) {
-			vld_printf(stderr, "%5d %s %c %c %c %s %-28s %s %-14s ", nr, VLD_G(col_sep), notdead ? ' ' : '*', start ? '>' : ' ', end ? '>' : ' ', VLD_G(col_sep), opcodes[op.opcode].name, VLD_G(col_sep), fetch_type);
+			vld_printf(stderr, "%5d %s %c %c %c %c %s %-28s %s %-14s ", nr, VLD_G(col_sep), notdead ? ' ' : '*', entry ? 'E' : ' ', start ? '>' : ' ', end ? '>' : ' ', VLD_G(col_sep), opcodes[op.opcode].name, VLD_G(col_sep), fetch_type);
 		} else {
-			vld_printf(stderr, "%5d%c %c %c %-28s %-14s ", nr, notdead ? ' ' : '*', start ? '>' : ' ', end ? '>' : ' ', opcodes[op.opcode].name, fetch_type);
+			vld_printf(stderr, "%5d%c %c %c %c %-28s %-14s ", nr, notdead ? ' ' : '*', entry ? 'E' : ' ', start ? '>' : ' ', end ? '>' : ' ', opcodes[op.opcode].name, fetch_type);
 		}
 	}
 
 	if (flags & EXT_VAL) {
-		vld_printf(stderr, "%3X  ", op.extended_value);
+		vld_printf(stderr, "%3d  ", op.extended_value);
 	} else {
 		vld_printf(stderr, "     ");
 	}
@@ -661,9 +695,9 @@ void vld_dump_op(int nr, zend_op * op_ptr, zend_uint base_address, int notdead,
 
 		VLD_PRINT(3, " BRK_CONT[ ");
 #if PHP_VERSION_ID >= 50399
-		el = vld_find_brk_cont(op.op2.constant, op.op1.opline_num, opa);
+		el = vld_find_brk_cont(Z_LVAL_P(op.op2.zv), VLD_ZNODE_ELEM(op.op1, opline_num), opa);
 #else
-		el = vld_find_brk_cont(op.op2.u.constant.value.lval, op.op1.u.opline_num, opa);
+		el = vld_find_brk_cont(op.op2.u.constant.value.lval, VLD_ZNODE_ELEM(op.op1, opline_num), opa);
 #endif
 		jmp = op.opcode == ZEND_BRK ? el->brk : el->cont;
 		vld_printf (stderr, ", ->%d", jmp);
@@ -684,43 +718,45 @@ void vld_dump_oparray(zend_op_array *opa TSRMLS_DC)
 	unsigned int i;
 	vld_set *set;
 	vld_branch_info *branch_info;
-	zend_uint base_address = (zend_uint) &(opa->opcodes[0]);
+	unsigned int base_address = (unsigned int)(zend_intptr_t)&(opa->opcodes[0]);
 
 	set = vld_set_create(opa->last);
 	branch_info = vld_branch_info_create(opa->last);
 
-	vld_analyse_oparray(opa, set, branch_info TSRMLS_CC);
+	if (VLD_G(dump_paths)) {
+		vld_analyse_oparray(opa, set, branch_info TSRMLS_CC);
+	}
 	if (VLD_G(format)) {
-		vld_printf (stderr, "filename:%s%s\n", VLD_G(col_sep), opa->filename);
-		vld_printf (stderr, "function name:%s" ZSTRFMT "\n", VLD_G(col_sep), ZSTRCP(opa->function_name));
+		vld_printf (stderr, "filename:%s%s\n", VLD_G(col_sep), ZSTRING_VALUE(opa->filename));
+		vld_printf (stderr, "function name:%s%s\n", VLD_G(col_sep), ZSTRING_VALUE(opa->function_name));
 		vld_printf (stderr, "number of ops:%s%d\n", VLD_G(col_sep), opa->last);
 	} else {
-		vld_printf (stderr, "filename:       %s\n", opa->filename);
-		vld_printf (stderr, "function name:  " ZSTRFMT "\n", ZSTRCP(opa->function_name));
+		vld_printf (stderr, "filename:       %s\n", ZSTRING_VALUE(opa->filename));
+		vld_printf (stderr, "function name:  %s\n", ZSTRING_VALUE(opa->function_name));
 		vld_printf (stderr, "number of ops:  %d\n", opa->last);
 	}
 #ifdef IS_CV /* PHP >= 5.1 */
 	vld_printf (stderr, "compiled vars:  ");
 	for (i = 0; i < opa->last_var; i++) {
-		vld_printf (stderr, "!%d = $" ZSTRFMT "%s", i, ZSTRCP(opa->vars[i].name), ((i + 1) == opa->last_var) ? "\n" : ", ");
+		vld_printf (stderr, "!%d = $%s%s", i, OPARRAY_VAR_NAME(opa->vars[i]), ((i + 1) == opa->last_var) ? "\n" : ", ");
 	}
 	if (!opa->last_var) {
 		vld_printf(stderr, "none\n");
 	}
 #endif
 	if (VLD_G(format)) {
-		vld_printf(stderr, "line%s# *%s%sop%sfetch%sext%sreturn%soperands\n",VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep));
+		vld_printf(stderr, "line%s# *%s%s%sop%sfetch%sext%sreturn%soperands\n",VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep),VLD_G(col_sep));
 	} else {
-		vld_printf(stderr, "line     # *  op                           fetch          ext  return  operands\n");
-		vld_printf(stderr, "---------------------------------------------------------------------------------\n");
+		vld_printf(stderr, "line     #* E I O op                           fetch          ext  return  operands\n");
+		vld_printf(stderr, "-------------------------------------------------------------------------------------\n");
 	}
 	for (i = 0; i < opa->last; i++) {
-		vld_dump_op(i, opa->opcodes, base_address, vld_set_in(set, i), vld_set_in(branch_info->starts, i), vld_set_in(branch_info->ends, i), opa TSRMLS_CC);
+		vld_dump_op(i, opa->opcodes, base_address, vld_set_in(set, i), vld_set_in(branch_info->entry_points, i), vld_set_in(branch_info->starts, i), vld_set_in(branch_info->ends, i), opa TSRMLS_CC);
 	}
 	vld_printf(stderr, "\n");
 
 	if (VLD_G(dump_paths)) {
-		vld_branch_post_process(branch_info);
+		vld_branch_post_process(opa, branch_info);
 		vld_branch_find_paths(branch_info);
 		vld_branch_info_dump(opa, branch_info TSRMLS_CC);
 	}
@@ -734,7 +770,7 @@ void opt_set_nop (zend_op_array *opa, int nr)
 	opa->opcodes[nr].opcode = ZEND_NOP;
 }
 
-zend_brk_cont_element* vld_find_brk_cont(zend_uint nest_levels, int array_offset, zend_op_array *op_array)
+zend_brk_cont_element* vld_find_brk_cont(int nest_levels, int array_offset, zend_op_array *op_array)
 {
 	zend_brk_cont_element *jmp_to;
 
@@ -771,7 +807,11 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 #endif
 		return 1;
 	} else if (opcode.opcode == ZEND_JMPZNZ) {
+#if PHP_VERSION_ID >= 70000
+		*jmp1 = VLD_ZNODE_ELEM(opcode.op2, jmp_addr) - base_address;
+#else
 		*jmp1 = VLD_ZNODE_ELEM(opcode.op2, opline_num);
+#endif
 		*jmp2 = opcode.extended_value;
 		return 1;
 	} else if (opcode.opcode == ZEND_BRK || opcode.opcode == ZEND_CONT) {
@@ -783,7 +823,7 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 #endif
 		) {
 #if PHP_VERSION_ID >= 50399
-			el = vld_find_brk_cont(opcode.op2.constant, VLD_ZNODE_ELEM(opcode.op1, opline_num), opa);
+			el = vld_find_brk_cont(Z_LVAL_P(opcode.op2.zv), VLD_ZNODE_ELEM(opcode.op1, opline_num), opa);
 #else
 			el = vld_find_brk_cont(opcode.op2.u.constant.value.lval, VLD_ZNODE_ELEM(opcode.op1, opline_num), opa);
 #endif
@@ -792,14 +832,37 @@ int vld_find_jump(zend_op_array *opa, unsigned int position, long *jmp1, long *j
 		}
 	} else if (opcode.opcode == ZEND_FE_RESET || opcode.opcode == ZEND_FE_FETCH) {
 		*jmp1 = position + 1;
+#if PHP_VERSION_ID >= 70000
+		*jmp2 = VLD_ZNODE_ELEM(opcode.op2, jmp_addr) - base_address;
+#else
 		*jmp2 = VLD_ZNODE_ELEM(opcode.op2, opline_num);
+#endif
+		return 1;
+	} else if (opcode.opcode == ZEND_CATCH) {
+		*jmp1 = position + 1;
+#if PHP_VERSION_ID >= 50400
+        if (!opcode.result.num) {
+#else
+        if (!opcode.op1.u.EA.type) {
+#endif
+            *jmp2 = opcode.extended_value;
+            if (*jmp2 == *jmp1) {
+                *jmp2 = VLD_JMP_NOT_SET;
+            }
+        } else {
+			*jmp2 = VLD_JMP_EXIT;
+		}
 		return 1;
 #if (PHP_MAJOR_VERSION > 5) || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
 	} else if (opcode.opcode == ZEND_GOTO) {
 		*jmp1 = ((long) VLD_ZNODE_ELEM(opcode.op1, jmp_addr) - (long) base_address) / sizeof(zend_op);
 		return 1;
 #endif
+	} else if (opcode.opcode == ZEND_EXIT || opcode.opcode == ZEND_THROW || opcode.opcode == ZEND_RETURN) {
+		*jmp1 = VLD_JMP_EXIT;
+		return 1;
 	}
+
 	return 0;
 }
 
@@ -811,6 +874,7 @@ void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *bran
 	while (position < opa->last) {
 		if (position == 0) {
 			vld_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			vld_set_add(branch_info->entry_points, position);
 #if PHP_MAJOR_VERSION >= 5
 		} else if (opa->opcodes[position].opcode == ZEND_CATCH) {
 			if (VLD_G(format)) {
@@ -819,6 +883,7 @@ void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *bran
 				VLD_PRINT1(1, "Found catch point at position: %d\n", position);
 			}
 			vld_analyse_branch(opa, position, set, branch_info TSRMLS_CC);
+			vld_set_add(branch_info->entry_points, position);
 #endif
 		}
 		position++;
@@ -829,9 +894,8 @@ void vld_analyse_oparray(zend_op_array *opa, vld_set *set, vld_branch_info *bran
 
 void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set, vld_branch_info *branch_info TSRMLS_DC)
 {
-	long jump_pos1 = -1;
-	long jump_pos2 = -1;
-	unsigned int branchnr;
+	long jump_pos1 = VLD_JMP_NOT_SET;
+	long jump_pos2 = VLD_JMP_NOT_SET;
 
 	if (VLD_G(format)) {
 		VLD_PRINT2(1, "Branch analysis from position:%s%d\n", VLD_G(col_sep),position);
@@ -850,22 +914,28 @@ void vld_analyse_branch(zend_op_array *opa, unsigned int position, vld_set *set,
 	VLD_PRINT1(2, "Add %d\n", position);
 	vld_set_add(set, position);
 	while (position < opa->last) {
-		jump_pos1 = -1;
-		jump_pos2 = -1;
+		jump_pos1 = VLD_JMP_NOT_SET;
+		jump_pos2 = VLD_JMP_NOT_SET;
 
 		/* See if we have a jump instruction */
 		if (vld_find_jump(opa, position, &jump_pos1, &jump_pos2)) {
 			VLD_PRINT1(1, "Jump found. Position 1 = %d", jump_pos1);
-			if (jump_pos2 != -1) {
+			if (jump_pos2 != VLD_JMP_NOT_SET) {
 				VLD_PRINT1(1, ", Position 2 = %d\n", jump_pos2);
 			} else {
 				VLD_PRINT(1, "\n");
 			}
-			vld_branch_info_update(branch_info, position, opa->opcodes[position].lineno, 0, jump_pos1);
-			vld_analyse_branch(opa, jump_pos1, set, branch_info TSRMLS_CC);
-			if (jump_pos2 != -1) {
+			if (jump_pos1 == VLD_JMP_EXIT || jump_pos1 >= 0) {
+				vld_branch_info_update(branch_info, position, opa->opcodes[position].lineno, 0, jump_pos1);
+				if (jump_pos1 != VLD_JMP_EXIT) {
+					vld_analyse_branch(opa, jump_pos1, set, branch_info TSRMLS_CC);
+				}
+			}
+			if (jump_pos2 == VLD_JMP_EXIT || jump_pos2 >= 0) {
 				vld_branch_info_update(branch_info, position, opa->opcodes[position].lineno, 1, jump_pos2);
-				vld_analyse_branch(opa, jump_pos2, set, branch_info TSRMLS_CC);
+				if (jump_pos2 != VLD_JMP_EXIT) {
+					vld_analyse_branch(opa, jump_pos2, set, branch_info TSRMLS_CC);
+				}
 			}
 			break;
 		}
