@@ -1,16 +1,10 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 4                                                        |
+   | Copyright (c) 1997-2013 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997, 1998, 1999, 2000, 2001 The PHP Group             |
-   +----------------------------------------------------------------------+
-   | This source file is subject to version 2.02 of the PHP license,      |
-   | that is bundled with this package in the file LICENSE, and is        |
-   | available at through the world-wide-web at                           |
-   | http://www.php.net/license/2_02.txt.                                 |
-   | If you did not receive a copy of the PHP license and are unable to   |
-   | obtain it through the world-wide-web, please send a note to          |
-   | license@php.net so we can mail you a copy immediately.               |
+   | This source file is subject to the 2-Clause BSD license which is     |
+   | available through the LICENSE file, or online at                     |
+   | http://opensource.org/licenses/bsd-license.php                       |
    +----------------------------------------------------------------------+
    | Authors:  Derick Rethans <derick@derickrethans.nl>                   |
    +----------------------------------------------------------------------+
@@ -69,7 +63,7 @@ zend_module_entry vld_module_entry = {
 	PHP_RSHUTDOWN(vld),
 	PHP_MINFO(vld),
 #if ZEND_MODULE_API_NO >= 20010901
-	"0.12.0",
+	"0.13.0",
 #endif
 	STANDARD_MODULE_PROPERTIES
 };
@@ -103,6 +97,9 @@ static void vld_init_globals(zend_vld_globals *vld_globals)
 	vld_globals->format       = 0;
 	vld_globals->col_sep	  = "\t";
 	vld_globals->path_dump_file = NULL;
+	vld_globals->dump_paths   = 1;
+	vld_globals->save_paths   = 0;
+	vld_globals->verbosity    = 1;
 }
 
 
@@ -254,13 +251,13 @@ static int vld_dump_fe (zend_op_array *fe APPLY_TSRMLS_DC, int num_args, va_list
 	TSRMLS_FETCH()
 #endif
 	if (fe->type == ZEND_USER_FUNCTION) {
-		char *new_str;
+		ZVAL_VALUE_STRING_TYPE *new_str;
 		int new_len;
 
-		new_str = php_url_encode(ZSTRKEY(hash_key->arKey), hash_key->nKeyLength - 1, &new_len);
-		vld_printf(stderr, "Function " ZSTRFMT ":\n", new_str);
+		new_str = php_url_encode(ZHASHKEYSTR(hash_key), ZHASHKEYLEN(hash_key) - 1 PHP_URLENCODE_NEW_LEN(new_len));
+		vld_printf(stderr, "Function %s:\n", ZSTRING_VALUE(new_str));
 		vld_dump_oparray(fe TSRMLS_CC);
-		vld_printf(stderr, "End of function " ZSTRFMT ".\n\n", new_str);
+		vld_printf(stderr, "End of function %s\n\n", ZSTRING_VALUE(new_str));
 		efree(new_str);
 	}
 
@@ -284,16 +281,16 @@ static int vld_dump_cle (zend_class_entry *class_entry TSRMLS_DC)
 
 	if (ce->type != ZEND_INTERNAL_CLASS) {	
 		if (VLD_G(path_dump_file)) {
-			fprintf(VLD_G(path_dump_file), "subgraph cluster_class_" ZSTRFMT " { label=\"class " ZSTRFMT "\";\n", ZSTRCP(ce->name), ZSTRCP(ce->name));
+			fprintf(VLD_G(path_dump_file), "subgraph cluster_class_%s { label=\"class %s\";\n", ZSTRING_VALUE(ce->name), ZSTRING_VALUE(ce->name));
 		}
 
 		zend_hash_apply_with_argument(&ce->function_table, (apply_func_arg_t) vld_check_fe, (void *)&have_fe TSRMLS_CC);
 		if (have_fe) {
-			vld_printf(stderr, "Class " ZSTRFMT ":\n", ZSTRCP(ce->name));
+			vld_printf(stderr, "Class %s:\n", ZSTRING_VALUE(ce->name));
 			zend_hash_apply_with_arguments(&ce->function_table APPLY_TSRMLS_CC, (apply_func_args_t) vld_dump_fe, 0);
-			vld_printf(stderr, "End of class " ZSTRFMT ".\n\n", ZSTRCP(ce->name));
+			vld_printf(stderr, "End of class %s.\n\n", ZSTRING_VALUE(ce->name));
 		} else {
-			vld_printf(stderr, "Class " ZSTRFMT ": [no user functions]\n", ZSTRCP(ce->name));
+			vld_printf(stderr, "Class %s: [no user functions]\n", ZSTRING_VALUE(ce->name));
 		}
 
 		if (VLD_G(path_dump_file)) {
@@ -315,14 +312,22 @@ static zend_op_array *vld_compile_file(zend_file_handle *file_handle, int type T
 	     (VLD_G(skip_append)  && PG(auto_append_file)  && PG(auto_append_file)[0]  && PG(auto_append_file)  == file_handle->filename)))
 	{
 		zval nop;
+#if PHP_VERSION_ID >= 70000
+		zend_op_array *ret;
+		ZVAL_STRINGL(&nop, "RETURN ;", 8);
+		ret = compile_string(&nop, "NOP" TSRMLS_CC);
+		zval_dtor(&nop);
+		return ret;
+#else
 		ZVAL_STRINGL(&nop, "RETURN ;", 8, 0);
-		return compile_string(&nop, "NOP" TSRMLS_CC);;
+		return compile_string(&nop, "NOP" TSRMLS_CC);
+#endif
 	}
 
 	op_array = old_compile_file (file_handle, type TSRMLS_CC);
 
 	if (VLD_G(path_dump_file)) {
-		fprintf(VLD_G(path_dump_file), "subgraph cluster_file_%08x { label=\"file %s\";\n", op_array, op_array->filename ? op_array->filename : "__main");
+		fprintf(VLD_G(path_dump_file), "subgraph cluster_file_%08x { label=\"file %s\";\n", op_array, op_array->filename ? ZSTRING_VALUE(op_array->filename) : "__main");
 	}
 	if (op_array) {
 		vld_dump_oparray (op_array TSRMLS_CC);
